#include "fft_win.h"
#include "fft_ears.h"
#include "whisper.h"

const float WINDOW_WIDTH = 300.;
const float WINDOW_HEIGHT = 200.;

const float MARGIN = 10.;

/* The following are used for sending a control signal from
 * some other app.  The other app looks for  "whiper port" and
 * write_port's data in the range (0, 255). 
 */
port_id whisper_port;
thread_id listen_thread;
bool keep_listening;

int32 __listen(void *arg)
{
	fft_scroller *scroll = (fft_scroller *)arg;
	float fval;
	int32 val;
	whisper_port = create_port(48, "whisper port");

	while (keep_listening) {

		if (read_port(whisper_port, &val, NULL, 0) != B_NO_ERROR)
			break;
		scroll->SetValue( max_c(((float)val/255.0)*SCROLL_RANGE, 1.0));
	}
}


fft_window::fft_window(BPoint loc) : 
	BWindow(BRect(loc,
				BPoint(loc.x + WINDOW_WIDTH, loc.y + WINDOW_HEIGHT)),
			"Whisper",
			B_TITLED_WINDOW, B_NOT_RESIZABLE|B_NOT_ZOOMABLE)
{
	Lock();					

	BRect frame = Bounds();
	background = new BView(frame, "background", B_FOLLOW_ALL, B_WILL_DRAW);
	background->SetViewColor(250,240,250);
	AddChild(background);

	BRect vRect;
	BScrollBar *sb;
	int32 view_width = (FFT_BAR_COUNT*2)+2;
	vRect.left = vRect.top = vRect.right = vRect.bottom = MARGIN;
	vRect.right += view_width;
	vRect.bottom += FFT_BAR_RANGE;
	
	srcView = new fft_view(vRect, "abcd");
	
	vRect.top += FFT_BAR_RANGE + MARGIN;
	vRect.bottom = vRect.top+B_H_SCROLL_BAR_HEIGHT;
		
	scroll = new fft_scroller(vRect, "whisper scroller", NULL,
							SCROLL_MIN, SCROLL_MAX, B_HORIZONTAL);
	scroll->SetValue(SCROLL_DEFAULT);
	ResizeTo(view_width+(MARGIN*2),
		(FFT_BAR_RANGE)+(MARGIN*3)+B_H_SCROLL_BAR_HEIGHT);

	background->AddChild(srcView);
	background->AddChild(scroll);

	((whisper *)be_app)->SetEars(new fft_ears(srcView));
	SetPulseRate(100000);

	Unlock();
	listen_thread = spawn_thread(__listen, "Listener",
		B_DISPLAY_PRIORITY, (void *)scroll);
	keep_listening = true;
	resume_thread(listen_thread);
}


bool fft_window::QuitRequested(void)
{
	keep_listening = false;
	be_app->Quit();
	return TRUE;
}

fft_scroller::fft_scroller(BRect frame,
							const char *name,
							BView *target,
							float min,
							float max,
							orientation direction) :
	BScrollBar(frame, name, target, min, max, direction)
{
}


void fft_scroller::ValueChanged(float newValue)
{
	float shift;
	fft_ears *ears = ((whisper *)be_app)->FftEars();
	if (ears) {
		ears->SetFiddleValue(newValue);
	}
}


const float BUTTON_HEIGHT = 14.;
const float BUTTON_WIDTH = 80.;
const int32 BUTTON_COUNT = 5;
const float INTER_BUTTON = 3.;

const float FIDDLE_WIDTH = (BUTTON_WIDTH + (INTER_BUTTON*2));
const float FIDDLE_HEIGHT = INTER_BUTTON + 
	((BUTTON_HEIGHT+INTER_BUTTON) * BUTTON_COUNT);

const uint32 SHIFT_MSG = 'shft';
const uint32 FLIPPER_MSG = 'flip';
const uint32 SPLATTER_MSG = 'splt';
const uint32 CONVOLVE_MSG = 'cnvl';
const uint32 WHISPER_MSG = 'wspr';

fiddle_window::fiddle_window(BPoint loc) : 
	BWindow(BRect(loc,
				BPoint(loc.x + FIDDLE_WIDTH, loc.y + FIDDLE_HEIGHT)),
			"Fiddle",
			B_TITLED_WINDOW, B_NOT_CLOSABLE|B_NOT_RESIZABLE|B_NOT_ZOOMABLE)
{
	BRect frame;
	float l,t;
	uint32 mult;
	Lock();		
	mult = 0;
	l = INTER_BUTTON;
	t = INTER_BUTTON+((BUTTON_HEIGHT + INTER_BUTTON)*mult);
	frame.Set(l,t,l+BUTTON_WIDTH,t+BUTTON_HEIGHT);
	BMessage *smsg = new BMessage(SHIFT_MSG);
	shift = new BRadioButton(frame, "Shift", "Shift", smsg);
	shift->SetValue(1);
	AddChild(shift);
	
	mult = 1;
	l = INTER_BUTTON;
	t = INTER_BUTTON+((BUTTON_HEIGHT + INTER_BUTTON)*mult);
	frame.Set(l,t,l+BUTTON_WIDTH,t+BUTTON_HEIGHT);
	BMessage *fmsg = new BMessage(FLIPPER_MSG);
	flipper = new BRadioButton(frame, "Flipper", "Flipper",fmsg);
	AddChild(flipper);

	mult = 2;
	l = INTER_BUTTON;
	t = INTER_BUTTON+((BUTTON_HEIGHT + INTER_BUTTON)*mult);
	frame.Set(l,t,l+BUTTON_WIDTH,t+BUTTON_HEIGHT);
	BMessage *spmsg = new BMessage(SPLATTER_MSG);
	splatter = new BRadioButton(frame, "Splatter", "Splatter",spmsg);
	AddChild(splatter);

	mult = 3;
	l = INTER_BUTTON;
	t = INTER_BUTTON+((BUTTON_HEIGHT + INTER_BUTTON)*mult);
	frame.Set(l,t,l+BUTTON_WIDTH,t+BUTTON_HEIGHT);
	BMessage *cvmsg = new BMessage(CONVOLVE_MSG);
	convolve = new BRadioButton(frame, "Whisper", "Whisper",cvmsg);
	AddChild(convolve);	

	mult = 4;
	l = INTER_BUTTON;
	t = INTER_BUTTON+((BUTTON_HEIGHT + INTER_BUTTON)*mult);
	frame.Set(l,t,l+BUTTON_WIDTH,t+BUTTON_HEIGHT);
	BMessage *wsmsg = new BMessage(WHISPER_MSG);
	wisper = new BRadioButton(frame, "Chirpy", "Chirpy",wsmsg);
	AddChild(wisper);	
	
	Unlock();
}

void fiddle_window::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case SHIFT_MSG:
			((whisper *)be_app)->FftEars()->SetFiddleMode(SHIFT);
			break;
		
		case FLIPPER_MSG:
			((whisper *)be_app)->FftEars()->SetFiddleMode(FLIPPER);
			break;	
		case SPLATTER_MSG:
			((whisper *)be_app)->FftEars()->SetFiddleMode(SPLATTER);
			break;		
		case CONVOLVE_MSG:
			((whisper *)be_app)->FftEars()->SetFiddleMode(CONVOLVE);
			break;		
		case WHISPER_MSG:
			((whisper *)be_app)->FftEars()->SetFiddleMode(WHISPER);
			break;	
		}
}

const uint32 SRATE_BUTTON_COUNT = 3;
const float SRATE_WIDTH = (BUTTON_WIDTH + (INTER_BUTTON*2));
const float SRATE_HEIGHT = INTER_BUTTON + 
	((BUTTON_HEIGHT+INTER_BUTTON) * SRATE_BUTTON_COUNT);

const uint32 S44_MSG = 's44';
const uint32 S22_MSG = 's22';
const uint32 S11_MSG = 's11';

srate_window::srate_window(BPoint loc) : 
	BWindow(BRect(loc,
				BPoint(loc.x + SRATE_WIDTH, loc.y + SRATE_HEIGHT)),
			"SRate",
			B_TITLED_WINDOW, B_NOT_CLOSABLE|B_NOT_RESIZABLE|B_NOT_ZOOMABLE)
{
	BRect frame;
	float l,t;
	uint32 mult;
	Lock();		
	mult = 0;
	l = INTER_BUTTON;
	t = INTER_BUTTON+((BUTTON_HEIGHT + INTER_BUTTON)*mult);
	frame.Set(l,t,l+BUTTON_WIDTH,t+BUTTON_HEIGHT);
	BMessage *s44msg = new BMessage(S44_MSG);
	s44 = new BRadioButton(frame, "44100", "44100", s44msg);
	s44->SetValue(1);
	AddChild(s44);
	
	mult = 1;
	l = INTER_BUTTON;
	t = INTER_BUTTON+((BUTTON_HEIGHT + INTER_BUTTON)*mult);
	frame.Set(l,t,l+BUTTON_WIDTH,t+BUTTON_HEIGHT);
	BMessage *s22msg = new BMessage(S22_MSG);
	s22 = new BRadioButton(frame, "22050", "22050",s22msg);
	AddChild(s22);

	mult = 2;
	l = INTER_BUTTON;
	t = INTER_BUTTON+((BUTTON_HEIGHT + INTER_BUTTON)*mult);
	frame.Set(l,t,l+BUTTON_WIDTH,t+BUTTON_HEIGHT);
	BMessage *s11msg = new BMessage(S11_MSG);
	s11 = new BRadioButton(frame, "11025", "11025",s11msg);
	AddChild(s11);

	Unlock();
}

void srate_window::MessageReceived(BMessage *msg)
{
	switch (msg->what) {

		case S44_MSG:
			((whisper *)be_app)->FftEars()->SetSRate(44100.);
			break;	
	
		case S22_MSG:
			((whisper *)be_app)->FftEars()->SetSRate(22050.);
			break;			

		case S11_MSG:
			((whisper *)be_app)->FftEars()->SetSRate(11025.);
			break;		
		}
}